package com.kingdee.eas.custom.synctask;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.bos.metadata.entity.EntityViewInfo;
import com.kingdee.bos.metadata.entity.FilterInfo;
import com.kingdee.bos.metadata.entity.FilterItemInfo;
import com.kingdee.bos.metadata.entity.SelectorItemCollection;
import com.kingdee.bos.metadata.query.util.CompareType;
import com.kingdee.eas.basedata.org.AdminOrgUnitCollection;
import com.kingdee.eas.basedata.org.AdminOrgUnitFactory;
import com.kingdee.eas.basedata.org.AdminOrgUnitInfo;
import com.kingdee.eas.basedata.org.IAdminOrgUnit;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.custom.log.ISyncLog;
import com.kingdee.eas.custom.log.SyncLogFactory;
import com.kingdee.eas.custom.log.SyncLogInfo;
import com.kingdee.eas.custom.log.app.DataDirectionEnum;
import com.kingdee.eas.custom.log.app.DockingSystemEnum;
import com.kingdee.eas.custom.log.app.SyncEntityNameEnum;
import com.kingdee.eas.custom.log.app.SyncStatusEnum;
import com.kingdee.eas.custom.synctask.entity.AdminOrg;
import com.kingdee.eas.hr.ats.AtsUtil;
import com.kingdee.util.StringUtils;
import okhttp3.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.log4j.Logger;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;


public class SyncTranForOAFacadeControllerBean extends AbstractSyncTranForOAFacadeControllerBean {
    private static Logger logger =
            Logger.getLogger(SyncTranForOAFacadeControllerBean.class);
    private Properties propt = new Properties();


    public SyncTranForOAFacadeControllerBean() throws BOSException {
        String syncOAConfigPath = System.getProperty("EAS_HOME") + "/server/properties/scy/syncOAConfig.properties";
        try {
            propt.load(new FileInputStream(syncOAConfigPath));
        } catch (IOException e) {
            e.printStackTrace();
            String errorMsg = "获取配置文件报错，请检查配置：" + syncOAConfigPath + "   " + e.getMessage();
            throw new BOSException(errorMsg);
        }
    }

    /**
     * 同步组织
     *
     * @param ctx
     * @param billds
     * @param action
     * @throws BOSException
     */
    @Override
    protected void _syncOrgUnitToOA(Context ctx, String billds, ActionTypeEnum action) throws BOSException, EASBizException {
        ISyncLog iSyncLog = SyncLogFactory.getLocalInstance(ctx);
        SyncLogInfo syncLogInfo = new SyncLogInfo();
        syncLogInfo.setEntityName(SyncEntityNameEnum.adminOrg);
        syncLogInfo.setDataDirection(DataDirectionEnum.outflow);
        syncLogInfo.setDockingSystem(DockingSystemEnum.OA);
        try {
            super._syncOrgUnitToOA(ctx, billds, action);
            logger.error("_syncOrgUnitToOA---start");
            logger.error("_syncOrgUnitToOA---billds: " + billds);
            IAdminOrgUnit iAdminOrgUnit = AdminOrgUnitFactory.getLocalInstance(ctx);
            List dataList = new ArrayList();
            //String actionType = action == null ? "" : action.getValue().toLowerCase();
            //当参数billIds不为空值时，增量同步参数中的部门数据到OA
            FilterInfo filterInfo = null;
            if (!StringUtils.isEmpty(billds)) {
                Set<String> adminOrgIds = AtsUtil.toSet(billds);
                filterInfo = new FilterInfo();
                filterInfo.getFilterItems().add(new FilterItemInfo("id", adminOrgIds, CompareType.INCLUDE));
            }
            SelectorItemCollection sic = new SelectorItemCollection();
            sic.add("*");
            sic.add("parent.number");
            EntityViewInfo entityViewInfo = EntityViewInfo.getInstance(filterInfo, sic, null);
            AdminOrgUnitCollection adminOrgUnitCollection = iAdminOrgUnit.getAdminOrgUnitCollection(entityViewInfo);
            for (int i = 0; i < adminOrgUnitCollection.size(); i++) {
                AdminOrgUnitInfo adminOrgUnitInfo = adminOrgUnitCollection.get(i);
                String orgName = adminOrgUnitInfo.getName();
                AdminOrg adminOrg = new AdminOrg();
                boolean isSealUp = adminOrgUnitInfo.isIsSealUp();
                if (isSealUp) {
                    adminOrg.setAction("delete");
                    adminOrg.setCanceled("0");
                } else {
                    adminOrg.setAction("add");
                }
                adminOrg.setCode(adminOrgUnitInfo.getNumber());
                adminOrg.setShortname(orgName);
                adminOrg.setFullname(orgName);
                //adminOrg.setParent_code("ITCS");
                adminOrg.setOrg_code("0");
                adminOrg.setParent_code(adminOrgUnitInfo.getParent().getNumber());
                //adminOrg.setOrder((String.valueOf(adminOrgUnitInfo.getIndex())));
                dataList.add(adminOrg);
            }
            //执行同步组织到OA
            executeSyncOrgUnitToOA(ctx, dataList, syncLogInfo, this.propt.getProperty("syncOrgUnitApiUrl"));
        } catch (Exception e) {
            e.printStackTrace();
            syncLogInfo.setSyncStatus(SyncStatusEnum.ERROR);
            syncLogInfo.setSyncResult(e.getMessage());
            iSyncLog.save(syncLogInfo);
            throw new BOSException(e.getMessage());
        }
    }


    /**
     * 执行同步组织到OA
     *
     * @param dataList
     * @return
     */
    public void executeSyncOrgUnitToOA(Context ctx, List dataList, SyncLogInfo syncLogInfo, String syncApiUrl) throws IOException, BOSException, EASBizException {
        if (dataList != null && dataList.size() > 0) {
            ObjectMapper mapper = new ObjectMapper();
            //封装请求参数
            String data = mapper.writeValueAsString(dataList);
            String params = String.format("token = %s &data = %s", getToken(), data);
            logger.error("executeSyncOrgUnitToOA----------------------params" + params);
            //封装请求参数
            //String syncOrgUnitApiUrl = "/api/hrm/resful/synDepartment";
            //String syncOrgUnitApiUrl = this.propt.getProperty("syncOrgUnitApiUrl");
            String url = this.propt.getProperty("url");
            logger.error("executeSyncOrgUnitToOA----------------------syncOrgUnitApiUrl" + syncApiUrl);
            if (StringUtils.isEmpty(syncApiUrl)) {
                throw new BOSException("同步组织到OA接口地址不能为空!");
            }
            OkHttpClient client = new OkHttpClient();
            MediaType mediaType = MediaType.parse("application/x-www-form-urlencoded");
            RequestBody body = RequestBody.create(mediaType, params);
            Request request = new Request.Builder()
                    .url(url + syncApiUrl)
                    .post(body)
                    .addHeader("Content-Type", "application/json")
                    .addHeader("Accept", "*/*")
                    .addHeader("Accept-Encoding", "gzip, deflate, br")
                    .addHeader("User-Agent", "PostmanRuntime-ApipostRuntime/1.1.0")
                    .addHeader("Connection", "keep-alive")
                    .build();
            Response response = client.newCall(request).execute();
            String respBody = response.body().string();
            if (response.isSuccessful()) {

            } else {

            }
        }
    }

    /**
     * 处理响应结果,写入日志
     *
     * @param ctx
     * @param response
     * @param syncLogInfo
     * @throws IOException
     * @throws BOSException
     * @throws EASBizException
     */
    private void handlerResponse(Context ctx, Response response, SyncLogInfo syncLogInfo)
            throws IOException, BOSException, EASBizException {
        if (syncLogInfo == null) {
            return;
        }
        ISyncLog iSyncLog = SyncLogFactory.getLocalInstance(ctx);
        if (response != null) {
            int success = 0;
            int fail = 0;
            ObjectMapper mapper = new ObjectMapper();
            String respBody = response.body().string();
            Map<String, Object> respMap = mapper.readValue(respBody, Map.class);
            if (response.isSuccessful()) {
                String code = (String) respMap.get("code");
                if ("1".equals(code)) {
                    StringBuilder msg = new StringBuilder();
                    for (String orgNumber : respMap.keySet()) {
                        Map<String, String> map = (Map<String, String>) respMap.get(orgNumber);
                        msg.append("[").append(orgNumber).append("]");
                        if ("1".equals(map.get("code"))) {
                            success++;
                            msg.append("同步成功\n");
                        } else {
                            fail++;
                            msg.append("同步失败，").append(map.get("msg")).append("\n");
                        }
                    }
                    syncLogInfo.setSyncStatus(SyncStatusEnum.SUCCESS);
                    syncLogInfo.setSyncCount(fail + success);
                    syncLogInfo.setSuccessNum(success);
                    syncLogInfo.setFailNum(fail);
                    syncLogInfo.setSyncResult(msg.toString());
                } else {
                    syncLogInfo.setSyncStatus(SyncStatusEnum.FAIL);
                    syncLogInfo.setSyncResult((String) respMap.get("msg"));
                }
            } else {
                syncLogInfo.setSyncStatus(SyncStatusEnum.FAIL);
                syncLogInfo.setSyncResult("请求超时");
            }
        } else {
            syncLogInfo.setSyncStatus(SyncStatusEnum.FAIL);
            syncLogInfo.setSyncResult("同步数据为空");
        }
        iSyncLog.save(syncLogInfo);
    }


    /**
     * 获取token
     *
     * @return
     */
    private String getToken() {
        String key = this.propt.getProperty("key");
        long l = System.currentTimeMillis();//时间戳毫秒数
        String code = key.concat(Long.toString(l));
        String s = DigestUtils.md5Hex(code).toUpperCase();
        Map<String, String> map = new HashMap<>();
        map.put("key", s);
        map.put("ts", Long.toString(l));
        logger.error("getToken" + JSONObject.toJSONString(map));
        return JSONObject.toJSONString(map);
    }

    /**
     * 同步职位信息
     *
     * @param ctx
     * @param billds
     * @param action
     * @throws BOSException
     */
    @Override
    protected void _syncPositionToOA(Context ctx, String billds, ActionTypeEnum action) throws BOSException, EASBizException {
        super._syncPositionToOA(ctx, billds, action);
    }

    /**
     * 同步员工信息
     *
     * @param ctx
     * @param billds
     * @param action
     * @throws BOSException
     */
    @Override
    protected void _syncPersonToOA(Context ctx, String billds, ActionTypeEnum action) throws BOSException, EASBizException {
        super._syncPersonToOA(ctx, billds, action);
    }

}
